local addon = LibStub("AceAddon-3.0"):GetAddon("Gratz")
local config = addon:NewModule("Gratz")

local db -- filled in OnInitialize

local function getOptions()
	local options = {
		type = "group",
		name = "Gratz",
		get = function(info) return db[info[#info]] end, -- info[#info] == name of option
		set = function(info, value) db[info[#info]] = value end,
		args = {
			about = {
				type = "description",
				name = "Gratz helps you keep tabs on your friends and guildmates.  It tells you when they level up, when they log on, when they log off, and so forth.",
				order = 0,
			},
		},
	}
	return options
end

local function getEventGrouping(name, order, private)
	local private_opt
	if private then 
		private_opt = {type = "toggle", name = "Privately", desc = "Just whisper the player (if applicable)", order = 21,}
	end
	local options = {
		type = "group",
		name = name,
		inline = true,
		order = order,
		args = {
			announce = {type = "toggle", name = "Announce", order = 10,},
			message = {type = "toggle", name = "Send message", order = 20,},
			private = private_opt,
			text = {type = "text", name = "Message text", order = 30,},
			threshold = {type = "range", name = "Threshold", desc = "The minimum level required for a message to be sent", min = 1, max = 80, step = 1, order = 40,},
		},
	}
	return options
end

local function getOptionSet(name, key, private)
	local options = {
		type = "group",
		name = name,
		get = function(info) return db[key][info[#info - 1]][info[#info]] end, -- info[#info] == name of option
		set = function(info, value) db[key][info[#info - 1]][info[#info]] = value end,
		args = {
			ding = getEventGrouping("Ding!", 10, private and true),
			connect = getEventGrouping("Connect", 20, private and true),
			disconnect = getEventGrouping("Disconnect", 30, private and false),
			add = getEventGrouping("Add", 40, private and true),
			remove = getEventGrouping("Remove", 50, private and true),
		},
	}
	return options
end

local function getGuild()
	return getOptionSet("Guild", "guild", true)
end

local function getFriends()
	return getOptionSet("Friends", "friends", false)
end

local function getOutput()
	local output = {
		name = "Message Output",
		type = "group",
		args = {
			desc = {
				type = "description", order = 0,
				name = "You can select where you want Gratz's messages displayed from this screen",
			},
			sink = addon:GetSinkAce3OptionsDataTable(),
		},
	}
	output.args.sink.order = 1
	output.args.sink.inline = true
	output.args.sink.name = ""
	return output
end

local function getProfiles()
	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
	return profiles
end

function config:OnInitialize()
	db = addon.db.profile

	local acreg = LibStub("AceConfigRegistry-3.0")
	local acdia = LibStub("AceConfigDialog-3.0")

	acreg:RegisterOptionsTable("Gratz", getOptions)
	acreg:RegisterOptionsTable("Gratz Friends", getFriends)
	acreg:RegisterOptionsTable("Gratz Guild", getGuild)
	acreg:RegisterOptionsTable("Gratz Output", getOutput)
	acreg:RegisterOptionsTable("Gratz Profiles", getProfiles)

	acdia:AddToBlizOptions("Gratz", "Gratz")
	acdia:AddToBlizOptions("Gratz Profiles", "Profiles", "Gratz")
	acdia:AddToBlizOptions("Gratz Output", "Output", "Gratz")
	acdia:AddToBlizOptions("Gratz Guild", "Guild", "Gratz")
	acdia:AddToBlizOptions("Gratz Friends", "Friends", "Gratz")

	local function openblizz() InterfaceOptionsFrame_OpenToFrame(acdia.BlizOptions["Gratz"].frame) end
	addon:RegisterChatCommand("gratz", openblizz)
end

